@@ -69,7 +69,7 @@ public:
     string=new Lum::String();
     string->SetFlex(false,false);
     string->SetMinWidth(Lum::Base::Size::stdCharWidth,6);
-    string->SetInputHelper(new Lum::InputHelperNumber(Lum::InputHelperNumber::typeFloat));
+    string->SetInputTypeHint(Lum::Object::inputTypeSignedFloatNumber);
 
     tmp=_(L"WEIGHT_EDIT_LABEL_WEIGHT",L"Weight ($unit):");
     Lum::Base::L10N::Substitute(tmp,L"$unit",weightUnit->Get());
@@ -136,7 +136,7 @@ public:
     string=new Lum::String();
     string->SetFlex(false,false);
     string->SetMinWidth(Lum::Base::Size::stdCharWidth,4);
-    string->SetInputHelper(new Lum::InputHelperNumber(Lum::InputHelperNumber::typeFloat));
+    string->SetInputTypeHint(Lum::Object::inputTypeSignedFloatNumber);
 
     tmp=_(L"FOOD_REGISTER_LABEL_AMOUNT",L"Amount ($unit):");
     Lum::Base::L10N::Substitute(tmp,L"$unit",unit);
@@ -178,55 +178,33 @@ public:
   }
 };
 
-class EntryObject : public Lum::Object
+class EntryModelPainter : public Lum::TableCellPainter
 {
 private:
-  Lum::OS::FontRef font;
-  const Entry      *entry;
-  size_t           dateWidth;
+  mutable size_t dateWidth;
 
 public:
-  EntryObject()
-  : font(Lum::OS::display->GetFont())
+  EntryModelPainter()
+  : dateWidth(0)
   {
-    dateWidth=0;
-    for (size_t i=0; i<=9; i++) {
-      dateWidth=std::max(dateWidth,font->StringWidth(Lum::Base::NumberToWString(i)));
+    // no code
+  }
+
+  void Draw(Lum::OS::DrawInfo* draw,
+            int x, int y, size_t width, size_t height) const
+  {
+    const Entry* entry=dynamic_cast<const EntryModel*>(GetModel())->GetEntry(GetRow());
+
+    if (dateWidth==0) {
+      for (size_t i=0; i<=9; i++) {
+        dateWidth=std::max(dateWidth,GetFont()->StringWidth(Lum::Base::NumberToWString(i)));
+      }
+
+      dateWidth=4*dateWidth+GetFont()->StringWidth(L":");
     }
 
-    dateWidth=4*dateWidth+font->StringWidth(L":");
-  }
-
-  void SetEntry(const Entry* entry)
-  {
-    this->entry=entry;
-  }
-
-  void CalcSize()
-  {
-    minWidth=0;
-    minHeight=font->height;
-
-    width=minWidth;
-    height=minHeight;
-
-    Object::CalcSize();
-  }
-
-  void Draw(int x, int y, size_t w, size_t h)
-  {
-    Object::Draw(x,y,w,h);
-
-    if (!OIntersect(x,y,w,h)) {
-      return;
-    }
-
-    /* --- */
-
-    Lum::OS::DrawInfo *draw=GetDrawInfo();
-
-    draw->PushFont(font);
-    draw->PushForeground(parent->GetTextColor(draw));
+    draw->PushFont(GetFont());
+    draw->PushForeground(GetTextColor(draw));
 
     //
     // Date of entry
@@ -245,8 +223,8 @@ public:
     date.append(Lum::Base::NumberToWString(entry->time.GetMinute()%10));
 
 
-    draw->DrawString(this->x,
-                     this->y+font->ascent,
+    draw->DrawString(x,
+                     y+GetFont()->ascent,
                      date);
 
     //
@@ -262,10 +240,10 @@ public:
       name=entry->item+L" ("+AmountToString(entry->amount)+L"x"+entry->unit+L")";
     }
 
-    draw->DrawString(this->x+
+    draw->DrawString(x+
                      dateWidth+
                      Lum::OS::display->GetSpaceHorizontal(Lum::OS::Display::spaceObjectBorder),
-                     this->y+font->ascent,
+                     y+GetFont()->ascent,
                      name);
 
     //
@@ -281,46 +259,13 @@ public:
       value=ValueToString((((Value)entry->amount)*entry->value)/10)+L" "+valueUnit->Get();
     }
 
-    draw->DrawString(this->x+width-font->StringWidth(value),
-                     this->y+font->ascent,
+    draw->DrawString(x+width-GetFont()->StringWidth(value),
+                     y+GetFont()->ascent,
                      value);
 
     draw->PopFont();
     draw->PopForeground();
   }
-
-  static EntryObject* Create(bool horizFlex, bool vertFlex)
-  {
-    EntryObject *object=new EntryObject();
-
-    object->SetFlex(horizFlex,vertFlex);
-
-    return object;
-  }
-};
-
-class EntryDataProvider : public EntryModel::DataProvider
-{
-private:
-  EntryObject *object;
-
-public:
-  EntryDataProvider()
-  : object(EntryObject::Create(true,false))
-  {
-    // no code
-  }
-
-  ~EntryDataProvider()
-  {
-    delete object;
-  }
-
-  Lum::Object* GetObject(const EntryModel::Iterator& iter, size_t column) const
-  {
-    object->SetEntry(*iter);
-    return object;
-  }
 };
 
 class EntryComparator : public EntryModel::Comparator
@@ -350,7 +295,7 @@ DataInput::DataInput()
 
    averageDiff(new Lum::Model::String(L"")),
    summary(new Lum::Model::String(L"")),
-   entries(new EntryModel(new EntryDataProvider(),new EntryComparator())),
+   entries(new EntryModel(new EntryComparator())),
    entrySelection(new Lum::Model::SingleLineSelection()),
 
    trend(new Lum::Model::Int(Trend::none))
@@ -442,6 +387,7 @@ void DataInput::CalcSize()
   table->SetSelection(entrySelection);
   table->SetDoubleClickAction(editItemAction);
   table->SetShowHeader(false);
+  table->SetPainter(new EntryModelPainter());
   table->GetTableView()->SetAutoFitColumns(true);
   table->GetTableView()->SetAutoHSize(true);
   et->Add(table);
