@@ -27,6 +27,11 @@
 #include <Lum/Model/String.h>
 #include <Lum/Model/Table.h>
 
+#include "Configuration.h"
+
+typedef Lum::Model::StdTable<Entry*>     EntryModel;
+typedef Lum::Base::Reference<EntryModel> EntryModelRef;
+
 class DataInput : public Lum::Component
 {
 private:
@@ -41,7 +46,7 @@ private:
   Lum::Model::StringRef              foodSearch;
   Lum::Model::StringRef              averageDiff;
   Lum::Model::StringRef              summary;
-  Lum::Model::ListTableRef           entries;
+  EntryModelRef                      entries;
   Lum::Model::SingleLineSelectionRef entrySelection;
   Lum::Model::IntRef                 trend;
 
