@@ -153,7 +153,7 @@ void Diagram::PrintData(Lum::OS::DrawInfo* draw,
   bool                                               finished=false;    // We have drawn one point pastthe visible end and can now finish
   int                                                px=0,py=0;         // Coordinates of previous point
   Lum::Base::Calendar                                day;
-  Value                                              lastWeight=0;
+  size_t                                             lastWeight=0;
 
   iter=data.data.lower_bound(start);
   while (iter!=data.data.begin()) {
@@ -395,14 +395,15 @@ void Diagram::PrintVerticalScale(Lum::OS::DrawInfo* draw)
   draw->PushFont(font);
 
   size_t step;
-  size_t effFontHeight;
+  int effFontHeight;
 
   if (font->height/4>2) {
     effFontHeight=font->height+(font->height/4);
   }
   else {
-  effFontHeight=font->height+2;
+    effFontHeight=font->height+2;
   }
+  assert(effFontHeight >= 0);
 
   // Weight on left side
 
@@ -499,9 +500,24 @@ void Diagram::SyncGraphWithDate()
   }
 }
 
-void Diagram::Draw(int x, int y, size_t w, size_t h)
+void Diagram::GetDimension(size_t& width, size_t& height)
 {
-  Lum::Scrollable::Draw(x,y,w,h);
+  size_t dayWidth=GetDayWidth();
+
+  width=std::max(width,data.GetDaysCovered()*dayWidth);
+}
+
+void Diagram::Layout()
+{
+  UpdateDimensions();
+
+  Scrollable::Layout();
+}
+
+void Diagram::Draw(Lum::OS::DrawInfo* draw,
+                   int x, int y, size_t w, size_t h)
+{
+  Lum::Scrollable::Draw(draw,x,y,w,h);
 
   if (!OIntersect(x,y,w,h)) {
     return;
@@ -516,7 +532,6 @@ void Diagram::Draw(int x, int y, size_t w, size_t h)
 
   /* --- */
 
-  Lum::OS::DrawInfo   *draw=GetDrawInfo();
   Lum::Base::Calendar start=data.GetStartDate();
   Lum::Base::Calendar end;
 
@@ -562,7 +577,7 @@ bool Diagram::HandleMouseEvent(const Lum::OS::MouseEvent& event)
   else if (event.IsGrabEnd() && event.button==Lum::OS::MouseEvent::button1) {
 
     if (std::abs(gestureStartX-event.x)>=std::abs(gestureStartY-event.y) ||
-        std::abs(gestureStartY-event.y)<(height*10)/100) {
+        std::abs(gestureStartY-event.y)<int((height*10)/100)) {
       return false;
     }
 
@@ -580,6 +595,7 @@ bool Diagram::HandleMouseEvent(const Lum::OS::MouseEvent& event)
       }
     }
 
+    UpdateDimensions();
     Redraw();
   }
   else if (event.type==Lum::OS::MouseEvent::down &&
@@ -592,6 +608,7 @@ bool Diagram::HandleMouseEvent(const Lum::OS::MouseEvent& event)
      mode=(Mode)((mode-1)%(modeMax+1));
    }
 
+   UpdateDimensions();
    Redraw();
 
    return true;
@@ -601,6 +618,7 @@ bool Diagram::HandleMouseEvent(const Lum::OS::MouseEvent& event)
           event.button==Lum::OS::MouseEvent::button5) {
    mode=(Mode)((mode+1)%(modeMax+1));
 
+   UpdateDimensions();
    Redraw();
 
    return true;
