@@ -80,7 +80,7 @@ public:
 
     string=new Lum::String();
     string->SetMinWidth(Lum::Base::Size::stdCharWidth,4);
-    string->SetInputHelper(new Lum::InputHelperNumber(Lum::InputHelperNumber::typeFloat));
+    string->SetInputTypeHint(Lum::Object::inputTypeSignedFloatNumber);
 
     tmp=_(L"FOOD_EDIT_LABEL_VALUE",L"Value ($unit):");
     Lum::Base::L10N::Substitute(tmp,L"$unit",valueUnit->Get());
@@ -146,7 +146,7 @@ public:
     string=new Lum::String();
     string->SetFlex(false,false);
     string->SetMinWidth(Lum::Base::Size::stdCharWidth,4);
-    string->SetInputHelper(new Lum::InputHelperNumber(Lum::InputHelperNumber::typeFloat));
+    string->SetInputTypeHint(Lum::Object::inputTypeSignedFloatNumber);
 
     tmp=_(L"FOOD_REGISTER_LABEL_AMOUNT",L"Amount ($unit):");
     Lum::Base::L10N::Substitute(tmp,L"$unit",unit);
@@ -188,57 +188,40 @@ public:
   }
 };
 
-class FoodObject : public Lum::Object
+class FoodModelPainter : public Lum::TableCellPainter
 {
 private:
-  Lum::OS::FontRef font;
-  const Food       *food;
+  Lum::OS::ColorRef goodFoodColor;
+  Lum::OS::ColorRef badFoodColor;
+  Lum::OS::FillRef  goodFoodBackground;
+  Lum::OS::FillRef  badFoodBackground;
 
 public:
-  FoodObject()
-  : font(Lum::OS::display->GetFont())
+  FoodModelPainter()
+   : goodFoodColor(0.82,1.0,0.82,Lum::OS::display->GetColor(Lum::OS::Display::whiteColor)),
+     badFoodColor(1.0,0.82,0.82,Lum::OS::display->GetColor(Lum::OS::Display::whiteColor))
   {
-    // no code
-  }
-
-  void SetFood(const Food* food)
-  {
-    this->food=food;
-  }
-
-  void CalcSize()
-  {
-    minWidth=0;
-    minHeight=font->height;
-
-    width=minWidth;
-    height=minHeight;
-
-    Object::CalcSize();
-  }
-
-  void Draw(int x, int y, size_t w, size_t h)
-  {
-    Object::Draw(x,y,w,h);
-
-    if (!OIntersect(x,y,w,h)) {
-      return;
+    if (!Lum::OS::display->GetTheme()->IsLightTheme()) {
+      goodFoodColor.Allocate(0.0,0.2,0.0,Lum::OS::display->GetColor(Lum::OS::Display::whiteColor));
+      badFoodColor.Allocate(0.2,0.0,0.0,Lum::OS::display->GetColor(Lum::OS::Display::whiteColor));
     }
 
-    /* --- */
+    goodFoodBackground=new Lum::OS::PlainFill(goodFoodColor);
+    badFoodBackground=new Lum::OS::PlainFill(badFoodColor);
+  }
 
-    Lum::OS::DrawInfo *draw=GetDrawInfo();
-
-    draw->PushFont(font);
-    draw->PushForeground(parent->GetTextColor(draw));
+  void Draw(Lum::OS::DrawInfo* draw,
+            int x, int y, size_t width, size_t height) const
+  {
+    const Food* food=dynamic_cast<const FoodModel*>(GetModel())->GetEntry(GetRow());
 
     //
     // Name of food
     //
 
-    draw->DrawString(this->x,
-                     this->y+font->ascent,
-                     food->name+L" ("+food->unit+L")");
+    DrawStringLeftAligned(draw,
+                          x,y,width,height,
+                          food->name+L" ("+food->unit+L")");
 
     //
     // Value + unit
@@ -246,45 +229,28 @@ public:
 
     std::wstring value=ValueToString(food->value)+L" "+valueUnit->Get();
 
-    draw->DrawString(this->x+width-font->StringWidth(value),
-                     this->y+font->ascent,
-                     value);
-
-    draw->PopFont();
-    draw->PopForeground();
+    DrawStringRightAligned(draw,
+                           x,y,width,height,
+                           value);
   }
 
-  static FoodObject* Create(bool horizFlex, bool vertFlex)
+  Lum::OS::Fill* GetCellBackground(Lum::TableView* table,
+                                   Lum::OS::DrawInfo* draw,
+                                   const Lum::Model::Table* model,
+                                   const Lum::Model::Selection* selection,
+                                   size_t column, size_t row) const
   {
-    FoodObject *object=new FoodObject();
+    const Food* food=dynamic_cast<const FoodModel*>(model)->GetEntry(row);
 
-    object->SetFlex(horizFlex,vertFlex);
-
-    return object;
-  }
-};
-
-class FoodDataProvider : public FoodModel::DataProvider
-{
-private:
-  FoodObject *object;
-
-public:
-  FoodDataProvider()
-  : object(FoodObject::Create(true,false))
-  {
-    // no code
-  }
-
-  ~FoodDataProvider()
-  {
-    delete object;
-  }
-
-  Lum::Object* GetObject(const FoodModel::Iterator& iter, size_t column) const
-  {
-    object->SetFood(*iter);
-    return object;
+    if (food->value<=(Value)goodValueLimit->Get()) {
+      return goodFoodBackground.Get();
+    }
+    else if (food->value>=(Value)badValueLimit->Get()) {
+      return badFoodBackground.Get();
+    }
+    else {
+      return NULL;
+    }
   }
 };
 
@@ -304,46 +270,6 @@ public:
   }
 };
 
-class FoodFormatProvider : public Lum::TableView::FormatProvider
-{
-private:
-  FoodModelRef      table;
-  Lum::OS::ColorRef goodFoodColor;
-  Lum::OS::ColorRef badFoodColor;
-  Lum::OS::FillRef  goodFoodBackground;
-  Lum::OS::FillRef  badFoodBackground;
-
-public:
-  FoodFormatProvider(FoodModel* table)
-   : table(table),
-     goodFoodColor(0.82,1.0,0.82,Lum::OS::display->GetColor(Lum::OS::Display::whiteColor)),
-     badFoodColor(1.0,0.82,0.82,Lum::OS::display->GetColor(Lum::OS::Display::whiteColor))
-  {
-    if (!Lum::OS::display->GetTheme()->IsLightTheme()) {
-      goodFoodColor.Allocate(0.0,0.2,0.0,Lum::OS::display->GetColor(Lum::OS::Display::whiteColor));
-      badFoodColor.Allocate(0.2,0.0,0.0,Lum::OS::display->GetColor(Lum::OS::Display::whiteColor));
-    }
-
-    goodFoodBackground=new Lum::OS::PlainFill(goodFoodColor);
-    badFoodBackground=new Lum::OS::PlainFill(badFoodColor);
-  }
-
-  Lum::OS::Fill* GetBackground(size_t column, size_t row) const
-  {
-    const Food* entry=table->GetEntry(row);
-
-    if (entry->value<=(Value)goodValueLimit->Get()) {
-      return goodFoodBackground.Get();
-    }
-    else if (entry->value>=(Value)badValueLimit->Get()) {
-      return badFoodBackground.Get();
-    }
-    else {
-      return Lum::TableView::FormatProvider::GetBackground(column,row);
-    }
-  }
-};
-
 FoodInput::FoodInput()
  : addFoodEntryAction(new Lum::Model::Action()),
    removeFoodEntryAction(new Lum::Model::Action()),
@@ -351,7 +277,7 @@ FoodInput::FoodInput()
    registerFoodAction(new Lum::Model::Action()),
 
    foodSearch(new Lum::Model::String(L"")),
-   foods(new FoodModel(new FoodDataProvider(),new FoodComparator())),
+   foods(new FoodModel(new FoodComparator())),
    foodSelection(new Lum::Model::SingleLineSelection()),
 
    searchTimerAction(new Lum::Model::Action())
@@ -426,9 +352,9 @@ void FoodInput::CalcSize()
   table->SetSelection(foodSelection);
   table->SetDoubleClickAction(registerFoodAction);
   table->SetShowHeader(false);
+  table->SetPainter(new FoodModelPainter());
   table->GetTableView()->SetAutoFitColumns(true);
   table->GetTableView()->SetAutoHSize(true);
-  table->GetTableView()->SetFormatProvider(new FoodFormatProvider(foods));
   ft->Add(table);
 
   menu=new Lum::Menu();
