@@ -34,15 +34,23 @@
 
 #include <Lum/Base/String.h>
 
+#include <Lum/Manager/FileSystem.h>
+
 void Import(Lum::Dialog *dialog)
 {
   Lum::Base::Path        path;
+  std::wstring           dir;
   std::wstring           file;
   Lum::Config::ErrorList errors;
   Lum::Config::Node      *top;
 
-  //path.SetNativeDirAndFile(Lum::Base::Path::GetHome(),L"WeightJinni-Export.xml");
-  path.SetNativeDir(Lum::Base::Path::GetHome());
+  if (!Lum::Manager::FileSystem::Instance()->GetEntry(Lum::Manager::FileSystem::userDocumentDir,
+                                                      dir)) {
+    Lum::Manager::FileSystem::Instance()->GetEntry(Lum::Manager::FileSystem::userDir,
+                                                   dir);
+  }
+
+  path.SetNativeDirAndFile(dir,L"WeightJinni-Export.xml");
 
   file=path.GetPath();
   if (!Lum::Dlg::File::GetFile(dialog->GetWindow(),
@@ -139,11 +147,17 @@ void Export(Lum::Dialog *dialog)
 {
   Lum::Config::Node *top;
   Lum::Base::Path   path;
+  std::wstring      dir;
   std::wstring      file;
   bool              res;
 
-  //path.SetNativeDirAndFile(Lum::Base::Path::GetHome(),L"WeightJinni-Export.xml");
-  path.SetNativeDir(Lum::Base::Path::GetHome());
+  if (!Lum::Manager::FileSystem::Instance()->GetEntry(Lum::Manager::FileSystem::userDocumentDir,
+                                                      dir)) {
+    Lum::Manager::FileSystem::Instance()->GetEntry(Lum::Manager::FileSystem::userDir,
+                                                   dir);
+  }
+
+  path.SetNativeDirAndFile(dir,L"WeightJinni-Export.xml");
 
   file=path.GetPath();
   if (!Lum::Dlg::File::SaveFile(dialog->GetWindow(),
